#!/bin/sh

###########################################################
#    Wirma V2 prod script                                 #
#                                                         #
#  $Id: prod_script.sh 1586 2014-03-10 14:37:36Z psa $   
#  Execution: SOURCED by initramfs at prod time           #
#                                                         #
#  Capacities:                                            #
#     * This script can use all the initramfs functions   #
#       and variables                                     #
#     * Must return 0 if all is OK                        #
###########################################################

### USE WITH ON DIE ECC FIRMWARE !!!


# defined in initrd's init:
#
# * remote_get <remote_file> <local_file>
# * prod_flash_"raw_image" <file>
#

# temp fdisk cmd
cmd=/tmp/fdisk_cmds


READ_TIMEOUT=20

set_mac()
{
	if [ `dmesg | grep -q "SoP-3G"` ]; then
		PREFIX="02:4B:03"
	else
		PREFIX="02:4B:08"
	fi

	echo -n "Enter last bytes of ethaddr: "
	read ADDR
	
	ETHADDR=`printf "%06X\n" 0x$ADDR | sed -r -e 's/(..)(..)(..)$/'${PREFIX}':\1:\2:\3/'`
	# Kerlink patch: add prod to fw_setenv to override ethaddr
	fw_setenv ethaddr "${ETHADDR}" prod
}


_rget()
{
	FILE=${1}
	DEST=${2:-"${TMP_DIR}/${FILE}"}

	remote_get ${DELIVERY_DIR}/${FILE} ${DEST}
	[ $? -ne 0 ] && info "Unable to retrieve ${FILE}" && return 1
	return 0
}

generic_flash()
{
	NAME="${1}"
	FILE="${2}"
	FUNCTION="${3}"

	remote_get ${DELIVERY_DIR}/${FILE} ${TMP_DIR}/${FILE}
	[ $? -ne 0 ] && info "Unable to retrieve ${NAME} (${FILE})" && return 1
	
	info "Flashing ${NAME}"
	${FUNCTION} ${TMP_DIR}/${FILE}
	[ $? -eq 0 ] && info "${NAME} programmed"
	echo ""
	rm -f ${TMP_DIR}/${FILE}
	return 0
}

flash_bootstrap()
{
	FILE="bootstrap.nanddump"
	
	_rget ${FILE}
	[ $? -ne 0 ] && return 1
	info "Flashing Bootstrap"
	# Burn Bootstrap
	nandwrite --scrub -o -n /dev/mtd0 ${TMP_DIR}/${FILE}
	[ $? -eq 0 ] && info "Bootstrap programmed"
	echo ""
	rm -f ${TMP_DIR}/${FILE}
	return 0
}

apply_dota()
{
	FILE="dota.tar.gz"
	
	mount -t yaffs2 ${USERFS_PART} ${MOUNT_POINT_USERFS} > /dev/null 2>&1 || error "Unable to mount ${MOUNT_POINT_USERFS}"
	mkdir -p ${MOUNT_POINT_USERFS}/dota
	_rget ${FILE} ${MOUNT_POINT_USERFS}/dota/${FILE}
	[ $? -ne 0 ] && return 1
	sync
	umount ${MOUNT_POINT_USERFS} > /dev/null 2>&1 || info "Unable to umount ${MOUNT_POINT_USERFS}"

	[ $? -ne 0 ] && return 2
	info "Applying Dota"
	dota
	echo ""
	return 0
}

flash_c8051()
{
	FILE="c8051.bin"

	remote_get "${DELIVERY_DIR}/${FILE}" "${TMP_DIR}/${FILE}"
	if [ $? -ne 0 ]
	then
		remote_get "${DELIVERY_DIR}/c8051/${FILE}" "${TMP_DIR}/${FILE}"
		[ $? -ne 0 ] && return $?
	fi
	info "Programming c8051"
	update_c8051 ${TMP_DIR}/${FILE}
	[ $? -eq 0 ] && info "c8051 programmed"
	echo ""
	rm -f ${TMP_DIR}/${FILE}
	return 0
}

flash_eeprom()
{
	CFG_FILE="${MOUNT_POINT_TOBEUPDATED}/etc/USB_uarts.cfg"
	
	info "Programming FTDI EEPROM"
	mount -t yaffs2 /dev/mtdblock5 ${MOUNT_POINT_TOBEUPDATED} 
	[ -f ${CFG_FILE} ] && update_FTDI_eeprom ${CFG_FILE}
	umount ${MOUNT_POINT_TOBEUPDATED}
	return 0
}

check_pic_version()
{
	PIC_VERSION=$(cat /sys/class/misc/klk-pic/version | sed -e 's/\./_/')
	FILE="w2m_${PIC_VERSION}.hex"

	info "Checking Pic version"
	remote_get "${DELIVERY_DIR}/Pic/${FILE}" "${TMP_DIR}/${FILE}"
	if [ $? -ne 0 ]
	then
		info " ***********************************************  "
	        info "    WARNING : Pic version flashed (${PIC_VERSION})" 
		info "          is NOT matching firmware one !!!        "
		info " ***********************************************  "
	else
		info "Pic version OK"
	fi
	return 0
}

ask_erase_mtd6()
{
	read -p "Erase User partition (mtd6) ? y/n [n] " -t ${READ_TIMEOUT} ENTRY
	if [ ${ENTRY} ] && [ ${ENTRY} = "y" ]
	then
		# PR 1741
		cat /dev/zero > /dev/mtd6
		flash_eraseall /dev/mtd6 > /dev/null 2>&1
	fi
}

ask_erase_mmc()
{
	read -p "Format SD Card ? y/n [n] " -t ${READ_TIMEOUT} ENTRY
	if [ ${ENTRY} ] && [ ${ENTRY} = "y" ]
	then
		# erase partition table
		echo -e "o\nw" > $cmd
		fdisk /dev/mmcblk0 < ${cmd} > /dev/null
		usleep 100

		# create partition table
		echo -e "n\np\n1\n \n " > $cmd
		echo "w" >> $cmd
		fdisk /dev/mmcblk0 < ${cmd}  > /dev/null
		usleep 500

		# format mmcblk0p1 with vfat
		if [ "${PRODUCT}" != "wirgrid" ]; then
			mkdosfs /dev/mmcblk0p1
		fi
	fi
}

get_and_flash_custo()
{
	custo_list="${1}"

	for custo in ${custo_list}
	do
		file="custo_$custo.tar.gz"
		remote_get custos/${file} ${TMP_DIR}/${file}
		if [ $? -ne 0 ] 
		then
			info "Unable to retrieve custo $file, retry with $custo."
			file="$custo"
			remote_get custos/${file} ${TMP_DIR}/${file}
			[ $? -ne 0 ] && info "Unable to retrieve custo $file" && return 1
		fi

		info "Flashing ${file}"
		prod_flash_custo ${TMP_DIR}/${file}
		[ $? -eq 0 ] && info "${file} programmed"
		echo ""
		rm -f ${TMP_DIR}/${file}
	done
	return 0	
}

get_and_flash_dota()
{
	dota_list="${1}"

	for dotaname in ${dota_list}
	do
		mount -t yaffs2 ${USERFS_PART} ${MOUNT_POINT_USERFS} > /dev/null 2>&1 || error "Unable to mount ${MOUNT_POINT_USERFS}"
		mkdir -p ${MOUNT_POINT_USERFS}/dota

		file="dota_${dotaname}.tar.gz"
		remote_get dotas/${file} ${MOUNT_POINT_USERFS}/dota/${file}
		if [ $? -ne 0 ] 
		then
			info "Unable to retrieve dota $file, retry with $dotaname."
			file="$dotaname"
			remote_get dotas/${file} ${MOUNT_POINT_USERFS}/dota/${file}
			[ $? -ne 0 ] && info "Unable to retrieve Dota ${file}" && return 1
		fi

		sync
		umount ${MOUNT_POINT_USERFS} > /dev/null 2>&1
		info "Applying Dota"
		dota
		[ $? -eq 0 ] && echo "Dota ${file} programmed"
		echo ""
	done
}

ask_custo()
{
	read -p "Add a custo ? <Enter XYZ if custo is named custo_XYZ.tar.gz> " -t ${READ_TIMEOUT} ENTRY
	if [ ${ENTRY} ]
	then
		get_and_flash_custo "custo_${ENTRY}.tar.gz"
	fi
	return $?
}

ask_dota()
{
	read -p "Add a dota ? <Enter XYZ if dota is named dota_XYZ.tar.gz> " -t ${READ_TIMEOUT} ENTRY
	if [ ${ENTRY} ]
	then
		get_and_flash_dota "dota_${ENTRY}.tar.gz"
	fi
	return $?
}


advanced_option_menu()
{
	while true
	do
		cat << EOF
 *****************************
 ***  Additional actions   ***
 *****************************
   Q - Quit
   C - Add Custo
   D - Add Dota
EOF
		[ ${USERFS_EMPTY} -ne 0 ] && echo "   U - Format User Partition"
		[ -d /sys/class/mmc_host/mmc0/mmc0:* ] && [ ${SDCARD_EMPTY} -eq 1 ] && echo "   S - Format SD card"
		
		echo ""
		entry=""
		read -p "Enter your choice > " -t ${READ_TIMEOUT} entry
		# If timeout, exit menu
		[ $? -ne 0 ] && return 1

		case "${entry}" in
			Q|q)
				return 0
				;;
			C|c)
				ask_custo
				;;
			D|d)
				ask_dota
				;;
			U|u)
				echo "Low level format user partition ..."
				flash_eraseall --scrub /dev/mtd6 > /dev/null 2>&1
				echo "User Partition is erased"
				;;
			S|s)
				echo "Formatting SD card"
				# erase partition table
				echo -e "o\nw" > $cmd
				fdisk /dev/mmcblk0 < ${cmd} > /dev/null
				usleep 100
		
				# create partition table
				echo -e "n\np\n1\n \n " > $cmd
				echo "w" >> $cmd
				fdisk /dev/mmcblk0 < ${cmd}  > /dev/null
				usleep 500
	
				# format mmcblk0p1 with vfat
				if [ "${PRODUCT}" != "wirgrid" ]; then
					mkdosfs /dev/mmcblk0p1
				fi
				echo "SD card is formatted"
				;;
		esac

	done
}


#########################
#######   MAIN  #########
#########################

echo "=========================================="
echo "====  Production Script starting   ======="
echo "=========================================="

# disable watchdog
wirma2hw wd down

ID=`fw_printenv ethaddr | sed 's/.*=//g' | awk -F: '{print $4$5$6}'`
if [ ${ID} == "FFFFFF" ]; then
	set_mac
fi

ID=`fw_printenv ethaddr | sed 's/.*=//g' | awk -F: '{print $1}'`
if [ ${ID} == "00" ]; then
	set_mac
fi

# Backup serialno in a specific variable
fw_setenv serialno `fw_printenv ethaddr | sed 's/.*=//g' | awk -F: '{print $3$4$5$6}'`

product=$(get_uvar product)
if [ -z "${product}" ]
then
  echo -n "Enter Product (road/rs/wirgrid/...): "
  read PRODUCT
  fw_setenv product "${PRODUCT}"

  echo -n "Enter Product PCB revision (4/5/...): "
  read PRODUCT_REV
  fw_setenv product_rev "${PRODUCT_REV}"
fi

#
# Check ON-Die ECC activated for Bootstrap flashing
# Pre first pass : no micondie -> force micondie=0
# First pass: micondie=0 -> flash bootstrap
# Second (real) pass : micondie=1 -> Flash all images
#
cat /proc/cmdline | grep -q "micondie=1"
if [ $? -ne 0 ]
then
	cat /proc/cmdline | grep -q "micondie=0"
	if [ $? -eq 0 ]
	then
		# On-Die ECC is forced for Bootstrap update
		flash_bootstrap

		# Tag who we are to come back after bootstrap update
		fw_setenv prod_script "${DELIVERY_DIR}/prod_script.sh"
		# Switch to new fw repo
		fw_setenv bootcmd "tftp 0x82000000 wirmaV2_ecc/prodfw/script_uboot.img; autoscr 0x82000000"
		# Force micondie on next reboot
		fw_setenv bootargs_prod "micondie=1"

		reboot -f
	else
		echo "Reboot without micondie for Boostrap update"
		# Tag who we are to come back after bootstrap update
		fw_setenv prod_script "${DELIVERY_DIR}/prod_script.sh"
		fw_setenv bootcmd "tftp 0x82000000 wirmaV2_ecc/prodfw/script_uboot.img; autoscr 0x82000000"
		fw_setenv bootargs_prod "micondie=0"
		reboot -f
	fi
fi


prod_quiet=$(get_uvar prod_quiet)

generic_flash "U-Boot" 	${UBOOT_IMAGE}			update_uboot

# Reflash Env to be sure
(
nanddump -o -b -n -f ${TMP_DIR}/uenv /dev/mtd2
flash_eraseall /dev/mtd2
nandwrite /dev/mtd2 ${TMP_DIR}/uenv
) > /dev/null 2>&1
info "U-Boot Env updated"
echo ""


generic_flash "Script" 	${SCRIPT_IMAGE}	 	prod_flash_script
generic_flash "Kernel" 	${KERNEL_IMAGE} 		prod_flash_kernel
generic_flash "Ramfs" 	${INITRAMFS_IMAGE} 	prod_flash_initramfs
generic_flash "Rootfs" 	${ROOTFS_IMAGE} 		prod_flash_rootfs

klklog "Production with `basename ${DELIVERY_DIR}`"

USERFS_EMPTY=1
# First try with TAR archive (quicker)
generic_flash "Userfs" 	${USERFS_IMAGE%.gz} 		update_userfs
if [ $? -eq 0 ]
then
	USERFS_EMPTY=0
else
	# Also try with gzipped archive (Slower)
	generic_flash "Userfs"  ${USERFS_IMAGE}      update_userfs
	[ $? -eq 0 ] && USERFS_EMPTY=0
fi

SDCARD_EMPTY=1
generic_flash "SD Card" 	${SDCARD_IMAGE} 	update_sdcard
[ $? -eq 0 ] && SDCARD_EMPTY=0

if [ ${prod_quiet} ]
then
	# In real production, if no userfs or SD card images, format them ... Before custo and dota install

	[ ${USERFS_EMPTY:=0} -eq 1 ] && flash_eraseall --scrub /dev/mtd6 > /dev/null 2>&1

	if [ -d /sys/class/mmc_host/mmc0/mmc0:* ] && [ ${SDCARD_EMPTY:=0} -eq 1 ]
	then
		# erase partition table
		echo -e "o\nw" > $cmd
		fdisk /dev/mmcblk0 < ${cmd} > /dev/null
		usleep 100
	
		# create partition table
		echo -e "n\np\n1\n \n " > $cmd
		echo "w" >> $cmd
		fdisk /dev/mmcblk0 < ${cmd}  > /dev/null
		usleep 500

		# format mmcblk0p1 with vfat
		if [ "${PRODUCT}" != "wirgrid" ]; then
			mkdosfs /dev/mmcblk0p1
		fi
	fi

else
	# Try to set RTC if PCF 8563 is detected
	if [ "rtc-pcf8563" == `cat /sys/class/rtc/rtc0/name` ] && [ -x /usr/sbin/ntpd ]
	then
		ntpd -q -g -x -p pool.ntp.org
		[ $? -eq 0 ] && hwclock -u -w
	fi
fi

if [ "${PRODUCT}" = "wirgrid" ]
then

	# erase userfs
	flash_eraseall --scrub /dev/mtd6 > /dev/null 2>&1

	# erase partition table
	echo -e "o\nw" > $cmd
	fdisk /dev/mmcblk0 < ${cmd} > /dev/null
	usleep 100
	
	# create partition table
	echo -e "n\np\n1\n \n " > $cmd
	echo "w" >> $cmd
	fdisk /dev/mmcblk0 < ${cmd}  > /dev/null
	usleep 500

	# format mmcblk0p1 with ext4
	mkfs.ext4 -O ^has_journal /dev/mmcblk0p1
fi

flash_c8051
[ -d "/sys/class/tty/ttyUSB0" ] && flash_eeprom

prod_custo=$(get_uvar prod_custo | sed 's/,/ /g')
[ -n "${prod_custo}" ] && get_and_flash_custo "${prod_custo}"

prod_dota=$(get_uvar prod_dota | sed 's/,/ /g')
[ -n "${prod_dota}" ] && get_and_flash_dota "${prod_dota}"

if ! [ ${prod_quiet} ]
then
	check_pic_version

	# Advanced User actions (format, add custo, dota, ...)
	advanced_option_menu
fi

# Clean U-boot env
fw_setenv prod_script
fw_setenv prod_user
fw_setenv prod_passwd
fw_setenv prod_custo
fw_setenv prod_dota
fw_setenv prod_quiet


fw_setenv bootargs_prod

# Fix bootdelay since rev 910 requires a password (and some modules are programmed in rev < 910)
fw_setenv bootdelay 1

# Added after delivery : max_bottfail = 21 instead of 5
fw_setenv max_bootfail 21

fw_setenv prod_fw `basename ${DELIVERY_DIR}`

# We are sourced so we must use return instead of exit
return 0
